
#include <Wire.h>
#include <Chronodot.h>
#include <MsTimer2.h>
#include <SD.h>

// 
//  
// Enerduino 
//  
// version 0.3
//  
// Internal revision: $Id: Enerduino.pde,v 0.22 2009/11/28 16:30:16 cesare Exp cesare $ 
//  
// written by Cesare Pizzi
// modified by Andrea Manzini
//  
// This simple Arduino application allow to monitor power consumpion by checking the flashing  
// light of the power meter. 
// There are 2 lights on italian ENEL power meters: RA (active) and RR (reactive). 
// Only RA led is computed for home contracts 
// One flash should be 1 w/h 
//  
// 

#include <stdlib.h> 
#include <stdarg.h>


#define DEBUG

/// The PIN to CS the SD card shield 
#define SD_CS_PIN 10 
// Analog input for photoresistor 
#define PHOTO_IN  0 

unsigned long timer=90000; // Log file is written every 15 minutes  FIXME 

unsigned long flash=0; 
int threshold=860;     // If photoresistor read more than this value,it count a flash 
int writeLog=0; 

Chronodot RTC;

// Arduino setup routine 
void setup(void)  
{ 
  
  // Setup for photoresistor 
  pinMode(PHOTO_IN,INPUT); 

  // Initialize timer 
  MsTimer2::set(timer, flushCounter); 
  MsTimer2::start(); 
  
  #ifdef DEBUG
  Serial.begin(9600);       // FIXME
  #endif
  
    SD.begin(SD_CS_PIN);  
    Wire.begin();
    RTC.begin();

  if (! RTC.isrunning()) {
    Serial.println("RTC is NOT running!");
    // following line sets the RTC to the date & time this sketch was compiled
    RTC.adjust(DateTime(__DATE__, __TIME__));
  }  
} 

// Main 
void loop(void)  
{ 

  //
  Serial.println(analogRead(PHOTO_IN));  // FIXME 

  // Read the photo sensor value 
  if (analogRead(PHOTO_IN) > threshold) 
  { 
    while (analogRead(PHOTO_IN) > threshold) 
    { 
      // Just wait the flash to turn off (to avoid multiple counts)    
    }  

    flash++; 
    #ifdef DEBUG
    Serial.print("DEBUG: Flash= ");       // FIXME
    Serial.println(flash);
    #endif 
  } 

  // Write the log file if interrupt has been called 
  if (writeLog==1) 
  { 
    DateTime now=RTC.now();
    uint16_t year=now.year();
    uint8_t month=now.month(); 
    uint8_t day=now.day();
    uint8_t hour=now.hour();
    uint8_t minute=now.minute();
    uint8_t second=now.second();
    String logStr=String(year,DEC);  
    if (month<10) logStr+='0';
    logStr+=month;
    if (day<10) logStr+='0';
    logStr+=day;
    if (hour<10) logStr+='0';
    logStr+=hour;
    if (minute<10) logStr+='0';
    logStr+=minute;
    if (second<10) logStr+='0';
    logStr+=second;
    logStr+=' ';
    logStr+=flash;
    File dataFile=SD.open("enerdata.txt",(O_APPEND | O_WRITE | O_CREAT));
    if (dataFile) { 
      
      #ifdef DEBUG
      Serial.println(logStr);
      #endif
     
      dataFile.println(logStr);
      dataFile.close();
    }
    writeLog=0; 
    flash=0;  
  } 

  delay(100); 
} 

///////////////// 
// Subroutines // 
///////////////// 

// Routine executed by the timer interrupt. This flush the  
// data to the log file 
void flushCounter(void) 
{ 
  writeLog=1; 
  #ifdef DEBUG
  Serial.println("timer irq");
  #endif
} 

